#include <Foundation/Foundation.h>
#include <AppKit/AppKit.h>
#include "AppController.h"
#include "FileRing.h"
#include "ImageWindow.h"

@implementation AppController

- (id) init
{
    [super init];
    
    _fileRing = nil;

    return self;
}

- (void) dealloc
{
    RELEASE(_fileRing);
    RELEASE(_window);
}

- (void) createMenu
{
    NSMenu * menu = AUTORELEASE( [NSMenu new] );

    [menu addItemWithTitle: @"Info Panel..."
          action: @selector(showInfoPanel:)
          keyEquivalent: @""];
    [menu addItemWithTitle: @"Open"
          action: @selector(open:)
          keyEquivalent: @"o"];
    [menu addItemWithTitle: @"Rotate"
          action: @selector(rotatePositive:)
          keyEquivalent: @"r"];
    [menu addItemWithTitle: @"Rotate back"
          action: @selector(rotateNegative:)
          keyEquivalent: @"R"];
    [menu addItemWithTitle: @"Next"
          action: @selector(nextImage:)
          keyEquivalent: @"n"];
    [menu addItemWithTitle: @"Previous"
          action: @selector(prevImage:)
          keyEquivalent: @"p"];
    [menu addItemWithTitle: @"Clip"
          action: @selector(clip:)
          keyEquivalent: @"c"];
    [menu addItemWithTitle: @"Undo"
          action: @selector(undo:)
          keyEquivalent: @"z"];
//     [menu addItemWithTitle: @"Save"
//           action: @selector(save:)
//           keyEquivalent: @"s"];
    [menu addItemWithTitle: @"Hide"
          action: @selector(hide:)
          keyEquivalent: @"h"];
    [menu addItemWithTitle:@"Quit"
          action:@selector(terminate:)
          keyEquivalent:@"q"];

    [NSApp setMainMenu: menu];
}


- (void) createWindow
{
    _window = [[ImageWindow alloc] init];
}


- (void) applicationWillFinishLaunching: (NSNotification *)not
{
    [self createMenu];
    [self createWindow];

    // get first command-line argument (zeroth is the program name)
    NSArray * args = [[NSProcessInfo processInfo] arguments];
    if ([args count] > 1)
        [self openFile: [args objectAtIndex:1] ];
}

- (void) applicationDidFinishLaunching: (NSNotification *)not
{
    [_window center];
    [_window makeKeyAndOrderFront: nil];
}

- (BOOL) applicationShouldTerminateAfterLastWindowClosed: (id)sender
{
    return YES;
}

- (void) open: (id)sender
{
    NSArray * image_types = [NSImage imageFileTypes];
    NSOpenPanel * panel = [NSOpenPanel openPanel];
    [panel setCanChooseDirectories: NO];

    NSString * dir = (_fileRing) ? [_fileRing directory] : NSHomeDirectory();

    NSString * fileName = (_fileRing) ? [_fileRing currentName] : nil;

    int result = [panel runModalForDirectory: dir
                        file: fileName
                        types: image_types ];
    if (result == NSOKButton)
    {
        NSArray * selection = [panel filenames];
        NSString * sel_file = [selection objectAtIndex: 0];
        if (sel_file)
            [self openFile: sel_file];
    }
}

- (void) openFile: (NSString *)fileName
{
    [self openFile: fileName updateRing: YES];
}

- (void) openFile: (NSString *)fileName updateRing: (BOOL) update
{
    NSAssert(_window, @"open: '_window' is not initialized");

    if (update)
    {
        RELEASE(_fileRing);
        _fileRing = [[FileRing alloc] initFromPath: fileName
                                      withTypes: [NSImage imageFileTypes]];
    }
    
    [_window setImageFromFile: fileName];
}

- (void) rotatePositive: (id)sender
{
    NSAssert(_window, @"rotatePositive: '_window' is not initialized");

    [_window rotateWithDirection: PHPositiveDirection];
}

- (void) rotateNegative: (id)sender
{
    NSAssert(_window, @"rotateNegative: '_window' is not initialized");

    [_window rotateWithDirection: PHNegativeDirection];
}

- (void) nextImage: (id)sender
{
    [self openFile: [_fileRing nextFile] updateRing: NO];
}

- (void) prevImage: (id)sender
{
    [self openFile: [_fileRing prevFile] updateRing: NO];
}

- (void) clip: (id)sender
{
    NSAssert(_window, @"clip: '_window' is not initialized");

    [_window clip];
}

- (void) undo: (id)sender
{
    NSAssert(_window, @"undo: '_window' is not initialized");

    [_window undo];
}

- (void) save: (id)sender
{
    NSAssert(_window, @"save: '_window' is not initialized");
    [_window saveToFile: @"tmpsave.tiff"];
}

- (void) showInfoPanel: (id)sender
{
    NSString * image_types = [[NSImage imageFileTypes] description];
    NSString * app_descr =
        [NSString stringWithFormat:
                      @"Simple image viewer. Reads %@", image_types];

    NSDictionary * options =
        [NSDictionary dictionaryWithObjectsAndKeys:
                      @"Simple image viewer", @"ApplicationDescription",
                      @"http://www.vaisburd.net/PhotoClip/", @"URL",
                      [NSArray arrayWithObject:
                                   @"Tima Vaisburd <tima@frys.com>"],
                      @"Authors",
                      @"0.2", @"ApplicationVersion",
                      @"Copyright (C) 2003 Tima Vaisburd", @"Copyright",
                      @"License: GNU GPL", @"CopyrightDescription",
                      nil];
                      
                                               
    [NSApp orderFrontStandardInfoPanelWithOptions: options];
}

@end
