#include "ImageWindow.h"
#include <AppKit/NSImageView.h>
#include <AppKit/NSImage.h>
#include <AppKit/NSScreen.h>

#include "PictureView.h"

#define APP_NAME "PhotoClip"
#define WIDTH_SCREEN_FRACTION (4./5.)
#define HEIGHT_SCREEN_FRACTION (4./5.)

@implementation ImageWindow

- (id) init
{
    // Determine good window size.
    NSSize screenSize = [[NSScreen mainScreen] frame].size;
    NSRect rect = NSMakeRect(0,0,
                             screenSize.width * WIDTH_SCREEN_FRACTION,
                             screenSize.height * HEIGHT_SCREEN_FRACTION);
    return [self initWithContentRect: rect];
}

- (id) initWithContentRect: (NSRect)rect
{
    unsigned int styleMask = NSTitledWindowMask
        | NSClosableWindowMask
        | NSMiniaturizableWindowMask
        | NSResizableWindowMask;

    [self initWithContentRect: rect
          styleMask: styleMask
          // backing: NSBackingStoreBuffered
          backing: NSBackingStoreRetained
          // backing: NSBackingStoreNonretained
          defer: YES];
    [self setTitle: @APP_NAME];

    NSView * view = AUTORELEASE ([PictureView new]);
    [view initWithFrame: rect];
    [self setContentView: view];

    [self setDelegate: view];
    
    return self;
}

- (void) setImageFromFile: (NSString *)fileName
{
    NSImage * newImage = [NSImage alloc];
    newImage = [newImage initWithContentsOfFile: fileName];
    if ( newImage == nil )
    {
        NSLog(@"can't load file '%@'\n", fileName);
    }
    
    [[self contentView] setImage: newImage];
    [newImage release];

    // Update the title
    
    NSString * title = @APP_NAME;
    NSString * fileNameShrt = [fileName lastPathComponent];

    if ( fileNameShrt != nil )
    {
        title = [title stringByAppendingFormat: @": %@", fileNameShrt];
        if ( newImage == nil )
            title = [title stringByAppendingString: @" (failed)"];
    }

    [self setTitle: title];

    
    // Print information about image

//     NSEnumerator * reps = [[newImage representations] objectEnumerator];
//     id rep;
//     while (rep = [reps nextObject])
//     {
//         fprintf(stderr, "rep=%@\n", [[rep class] description]);
//     }
}
 
- (void) rotateWithDirection: (PHRotateDirection) direction
{
    [[self contentView] rotateWithDirection: direction];
    [[self contentView] setNeedsDisplay:YES];
}

- (void) clip
{
    [[self contentView] clip];
}

- (void) undo
{
    [[self contentView] undo];
}

- (BOOL) saveToFile: (NSString *) path
{
    return [[self contentView] saveSceneToFile: path];
}

@end
