#ifndef _INCLUDED_Selection_H
#define _INCLUDED_Selection_H

#include <Foundation/NSObject.h>
#include <Foundation/NSGeometry.h>

@class NSAffineTransform;
@class NSEvent;
@class NSImage;

@interface Selection : NSObject
{
    NSRect  _fragment;      // In the image coordinates

    NSRect  _sketch;        // In the sketch coordinates
    NSPoint _start;
    NSPoint _end;
    NSRect _previousSelection;

    // Transformation fragment -> sketch
    NSAffineTransform   * _xform;
}

- (id) init;
- (id) initWithRect: (NSRect) fragment
              Xform: (NSAffineTransform *) xform // fragment -> sketch
              Start: (NSPoint) start;

- (void) setFragment: (NSRect) fragment xform: (NSAffineTransform *) xform;

// 'pt' is in sketch coordinates
- (void) mouseDown: (NSPoint) pt;
- (void) mouseDragged: (NSPoint) pt;
- (void) mouseUp: (NSPoint) pt;

// Returns YES if selection is non-empty
- (BOOL) valid;

// Sketch coordinates
- (NSRect) userSelRect;

// Sketch coordinates
- (NSRect) selectedInSketch;

// Sketch coordinates
- (NSRect) updateInSketch;

// Image coordinates
- (NSRect) selectedInImage;

- (NSImage *) mask;

// 'rect' is in sketch coordinates
- (NSImage *) maskForRect: (NSRect) rect;

@end // interface Selection

#endif // _INCLUDED_Selection_H
